@@ -19,8 +19,8 @@ use crate::idb::{
     Atom, Comparison, ComparisonOperator, Literal, Query, Rule as DlRule, Term, Variable,
 };
 use crate::io::{string_to_format, FilePragma, Format};
-use crate::syntax::{RESERVED_BOOLEAN_TRUE, RESERVED_PREFIX, TYPE_NAME_CONST_INTEGER};
-use crate::{relation_does_not_exist, Collection, Program};
+use crate::syntax::{TYPE_NAME_CONST_BOOLEAN, TYPE_NAME_CONST_INTEGER};
+use crate::{error, relation_does_not_exist, Collection, Program};
 use pest::iterators::{Pair, Pairs};
 use pest::{Parser, Span};
 use pest_derive::Parser;
@@ -426,7 +426,7 @@ fn parse_decl_inferred_relation(
                 if let Some(from) = program.extensional().get(&name) {
                     attributes.extend(from.schema().iter().cloned());
                 } else {
-                    return Err(relation_does_not_exist(name.as_ref().clone()));
+                    return Err(relation_does_not_exist(name));
                 }
             }
             _ => unreachable!("{:?}: {}", inner_pair.as_rule(), inner_pair.as_str()),
@@ -513,7 +513,7 @@ fn parse_decl_file_io(
 ) -> Result<()> {
     let first = input_pairs.next().unwrap();
     let predicate = match first.as_rule() {
-        Rule::predicate => Predicate::from_str_unchecked(first.as_str()),
+        Rule::predicate => program.predicates().fetch(first.as_str())?,
         _ => unreachable!(first.as_str()),
     };
 
@@ -715,9 +715,9 @@ fn parse_constant(
                 .into()
             // }
         }
-        Rule::boolean => (first.as_str()
-            == format!("{}{}", RESERVED_PREFIX, RESERVED_BOOLEAN_TRUE).as_str())
-        .into(),
+        Rule::boolean => bool::from_str(first.as_str())
+            .map_err(|_| error::invalid_value(TYPE_NAME_CONST_BOOLEAN, first.as_str()))?
+            .into(),
         _ => unreachable!(first.as_str()),
     };
 
@@ -735,7 +735,8 @@ fn parse_term(
 ) -> Result<Term> {
     let first = input_pairs.next().unwrap();
     let value = match first.as_rule() {
-        Rule::variable => Variable::from_str_unchecked(first.as_str()).into(),
+        Rule::anonymous_variable => Term::Anonymous,
+        Rule::named_variable => Variable::from_str_unchecked(first.as_str()).into(),
         Rule::constant => Term::Constant(parse_constant(first.into_inner(), program, features)?),
         _ => unreachable!(first.as_str()),
     };
